# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Utilities for migrating past Invenio versions to Invenio 3.0.

Invenio-Migrator consists of two parts:

- **Dumpers:** Tools for dumping data from an existing Invenio 1.2 or 2.x
  installation into a JSON file.
- **Loaders:** Tools for loading an Invenio legacy dump into an Invenio 3
  installation.

Dumping data
------------
First install Invenio-Migrator on your legacy installation (see
:ref:`install`). You now the ``invenio-migrator`` CLI availble to dump your
data:

.. code-block:: console

   $ inveniomigrator --help
   Usage: inveniomigrator [OPTIONS] COMMAND [ARGS]...

     Command for dumping legacy installation into a file JSON.

   Options:
     --help  Show this message and exit.

   Commands:
     dump  Dump data from Invenio legacy.


Records/files
~~~~~~~~~~~~~
You can use the CLI to dump e.g. records and files:

.. code-block:: console

   $ inveniomigrator dump records --query collection:preprints

This will dump the records and files in chunks of 1000 records into JSON files
in the current working directory. For each record Invenio-Migrator will dump:

- Record revisions in MARCXML and JSON
- Files for a record.

Loading data
------------
First install Invenio-Migrator on your Invenio 3 installation (see
:ref:`install`). This adds a ``dumps`` command to your instance CLI, which you
can use to load the record dumps:

.. code-block:: console

   $ <instance cmd> dumps loadrecords /path/to/records_dump_0.json

By default the MARCXML for each record is used to for loading the record. If
you wish to use the JSON for each record instead, simply use the
``--source-type json`` option to above command.


Customizing loading
-------------------
In case you need custom loading behavior of your dumps, you can subclass the
two classes :py:data:`invenio_migrator.records.RecordDump` and
:py:data:`invenio_migrator.records.RecordDumpLoader`. See API documentation for
further details.
"""

from __future__ import absolute_import, print_function

from .version import __version__

__all__ = ('__version__', 'InvenioMigrator')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Pytest configuration."""

from __future__ import absolute_import, print_function

import json
import os
import shutil
import tempfile
import uuid
from os.path import dirname, join

import pytest
from flask import Flask
from flask_babelex import Babel
from flask_celeryext import FlaskCeleryExt
from flask_cli import FlaskCLI, ScriptInfo
from invenio_accounts import InvenioAccounts
from invenio_accounts.models import User
from invenio_communities import InvenioCommunities
from invenio_db import db as db_
from invenio_db import InvenioDB
from invenio_deposit import InvenioDeposit
from invenio_files_rest import InvenioFilesREST
from invenio_files_rest.models import Bucket, Location, ObjectVersion
from invenio_jsonschemas.ext import InvenioJSONSchemas
from invenio_pidstore import InvenioPIDStore
from invenio_pidstore.fetchers import FetchedPID
from invenio_pidstore.models import PersistentIdentifier, PIDStatus, \
    RecordIdentifier
from invenio_pidstore.resolver import Resolver
from invenio_records import InvenioRecords
from invenio_records.api import Record
from invenio_sipstore.ext import InvenioSIPStore
from six import BytesIO
from sqlalchemy_utils.functions import create_database, database_exists

from invenio_migrator.ext import InvenioMigrator
from invenio_migrator.records import RecordDump

"""Pytest configuration."""


@pytest.yield_fixture(scope='session', autouse=True)
def app(request):
    """Flask application fixture."""
    app_ = Flask('testapp')
    app_.config.update(
        CELERY_ALWAYS_EAGER=True,
        CELERY_RESULT_BACKEND="cache",
        CELERY_CACHE_BACKEND="memory",
        CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
        TESTING=True,
        SQLALCHEMY_TRACK_MODIFICATIONS=True,
        SQLALCHEMY_DATABASE_URI=os.environ.get(
            'SQLALCHEMY_DATABASE_URI',
            'sqlite:///:memory:'),
        SECURITY_PASSWORD_SALT='TEST',
    )
    FlaskCLI(app_)
    FlaskCeleryExt(app_)
    InvenioDB(app_)
    InvenioRecords(app_)
    InvenioDeposit(app_)
    InvenioJSONSchemas(app_)
    InvenioAccounts(app_)
    InvenioCommunities(app_)
    InvenioPIDStore(app_)
    InvenioSIPStore(app_)
    Babel(app_)
    InvenioFilesREST(app_)
    InvenioMigrator(app_)

    with app_.app_context():
        yield app_


@pytest.fixture
def script_info(app):
    """Get ScriptInfo object for testing CLI."""
    return ScriptInfo(create_app=lambda info: app)


@pytest.yield_fixture()
def db(app):
    """Setup database."""
    if not database_exists(str(db_.engine.url)):
        create_database(str(db_.engine.url))
    db_.create_all()
    yield db_
    db_.session.remove()
    db_.drop_all()


@pytest.yield_fixture()
def dummy_location(db):
    """File system location."""
    tmppath = tempfile.mkdtemp()

    loc = Location(
        name='testloc',
        uri=tmppath,
        default=True
    )
    db.session.add(loc)
    db.session.commit()

    yield loc

    shutil.rmtree(tmppath)


@pytest.fixture()
def datadir():
    """Get data directory."""
    return join(dirname(__file__), 'data')


@pytest.fixture()
def logos_dir(datadir):
    """Get data directory."""
    return join(datadir, 'community_logos')


@pytest.fixture()
def records_json(datadir):
    """Load records json."""
    with open(join(datadir, 'records.json')) as fp:
        records = json.load(fp)
    return records


@pytest.fixture()
def communities_dump(datadir):
    """Load test data of dumped communities.

    :returns: Loaded dump of communities as a list of dict.
    :rtype: list
    """
    with open(join(datadir, 'communities.json')) as fp:
        records = json.load(fp)
    return records


@pytest.fixture()
def deposit_user(db):
    """Test user for deposit loading."""
    u1 = User(id=1, email='user@invenio.org',
              password='change_me', active=True)

    db.session.add(u1)
    db.session.commit()
    return u1


@pytest.fixture()
def deposit_record_pid(db):
    """Test record PID for deposit loading."""
    rec = Record.create({'title': 'Test'})
    rec_pid = PersistentIdentifier.create(
        pid_type='recid',
        pid_value='10',
        object_type='rec',
        object_uuid=rec.id,
        status=PIDStatus.REGISTERED,
    )
    db.session.commit()
    return rec_pid


@pytest.fixture()
def deposit_dump(datadir):
    """Load test data of dumped deposits.

    :returns: Loaded dump of deposits as a list of dict.
    :rtype: list
    """
    with open(join(datadir, 'deposit.json')) as fp:
        records = json.load(fp)
    return records


@pytest.fixture()
def featured_dump(datadir):
    """Load test data of dumped community featurings.

    :returns: Loaded dump of community featurings as a list of dict.
    :rtype: list
    """
    with open(join(datadir, 'featured.json')) as fp:
        records = json.load(fp)
    return records


@pytest.fixture()
def users_dump(datadir):
    """Load test data of dumped users.

    :returns: Loaded dump of users as a list of dict.
    :rtype: list
    """
    with open(join(datadir, 'users.json')) as fp:
        records = json.load(fp)
    return records


@pytest.fixture()
def record_db(db):
    """Load records json."""
    id_ = uuid.uuid4()
    record = Record.create({
        'title': 'Test record',
        'recid': 11782
    }, id_=id_)
    PersistentIdentifier.create(
        pid_type='recid',
        pid_value='11782',
        status=PIDStatus.REGISTERED,
        object_type='rec',
        object_uuid=id_,
    )
    RecordIdentifier.insert(11782)
    db.session.commit()
    return record


@pytest.fixture()
def record_pid(db, record_db):
    """Create record pid."""
    pid = PersistentIdentifier.create(
        pid_type='doi',
        pid_value='10.5281/zenodo.11782',
        status=PIDStatus.REGISTERED,
        object_type='rec',
        object_uuid=record_db.id,
    )
    db.session.commit()
    return pid


@pytest.fixture()
def record_file(db, dummy_location):
    """Create record pid."""
    data = b'testfile'
    b = Bucket.create()
    obj = ObjectVersion.create(
        b, 'CERN_openlab_Parin_Porecha.pdf', stream=BytesIO(data))
    db.session.commit()
    return dict(
        bucket=str(obj.bucket_id),
        key=obj.key,
        size=obj.file.size,
        checksum=obj.file.checksum,
    )


@pytest.fixture()
def record_dump(records_json):
    """A record dump."""
    def doi_fetcher(record_uuid, data):
        if 'doi' in data:
            return FetchedPID(
                pid_type='doi', pid_value=data['doi'], provider=None
            )
        return None
    return RecordDump(
        records_json[0], source_type='json', pid_fetchers=[doi_fetcher])


@pytest.fixture()
def resolver():
    """PID resolver."""
    return Resolver(
        pid_type='recid', object_type='rec', getter=Record.get_record)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

from datetime import datetime

import pytest
from invenio_files_rest.models import Bucket, BucketTag, FileInstance, \
    ObjectVersion
from invenio_pidstore.models import PersistentIdentifier, PIDStatus, \
    RecordIdentifier
from invenio_records.api import Record
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import NoResultFound

from invenio_migrator.records import RecordDumpLoader


def test_new_record(app, db, dummy_location, record_dump, resolver):
    """Test creation of new record."""
    RecordDumpLoader.create(record_dump)
    pid, record = resolver.resolve('11782')
    # Basic some test that record exists
    assert record['title']
    assert record.created == datetime(2014, 10, 13, 8, 27, 47)
    # Test that this is a completely new record
    assert len(record.revisions) == 2
    pytest.raises(IntegrityError, RecordIdentifier.insert, 11782)
    # Test the PIDs are extracted and created
    assert PersistentIdentifier.get('doi', '10.5281/zenodo.11782')

    assert len(record['_files']) == 1
    f = record['_files'][0]
    obj = ObjectVersion.get(f['bucket'], f['key'])
    assert obj.file.checksum == f['checksum']
    assert obj.file.size == f['size']

    assert BucketTag.get_value(f['bucket'], 'record') == str(record.id)


def test_update_record(app, db, dummy_location, record_dump, record_db,
                       resolver, record_file):
    """Test update of a record."""
    # Smoke test
    record_db['_files'] = [record_file]
    record_db.commit()
    db.session.commit()

    pytest.raises(IntegrityError, RecordIdentifier.insert, 11782)
    # Update record instead of create a new
    RecordDumpLoader.create(record_dump)
    pid, record = resolver.resolve('11782')
    # Basic some test that record exists
    assert record['title']
    assert record.created == datetime(2014, 10, 13, 8, 27, 47)
    # Test that old revisions are kept
    assert len(record.revisions) == 4
    # Test the PIDs are extracted and created
    assert PersistentIdentifier.get('doi', '10.5281/zenodo.11782')

    assert Bucket.query.count() == 1
    assert ObjectVersion.query.filter_by(is_head=True).count() == 1
    assert FileInstance.query.count() == 2

    assert len(record['_files']) == 1
    f = record['_files'][0]
    obj = ObjectVersion.get(f['bucket'], f['key'])
    assert obj.file.checksum != record_file['checksum']
    assert obj.file.size != record_file['size']


def test_deleted_record(app, db, dummy_location, record_dump, resolver):
    """Test init."""
    record_dump.data['record'][0]['json']['collections'] = ['deleted']
    record_dump.prepare_revisions()
    assert record_dump.is_deleted()

    RecordDumpLoader.create(record_dump)

    pid = PersistentIdentifier.get('recid', '11782')
    assert pid.status == PIDStatus.DELETED
    pid = PersistentIdentifier.get('doi', '10.5281/zenodo.11782')
    assert pid.status == PIDStatus.DELETED

    Record.get_record(pid.object_uuid, with_deleted=True)
    pytest.raises(
        NoResultFound, Record.get_record, pid.object_uuid)

    assert Bucket.query.count() == 1
    assert ObjectVersion.query.count() == 1
    assert FileInstance.query.count() == 1

    bucket = Bucket.query.all()[0]
    assert bucket.deleted



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Record and BibDocFile dump functions."""

from __future__ import absolute_import, print_function

import datetime
import zlib

from .bibdocfile import dump_bibdoc, get_modified_bibdoc_recids
from .utils import datetime_toutc


def _get_modified_recids_invenio12(from_date):
    """Get record ids for Invenio 1."""
    from invenio.search_engine import search_pattern
    from invenio.dbquery import run_sql
    return set((id[0] for id in run_sql(
        'select id from bibrec where modification_date >= %s',
        (from_date, ), run_on_slave=True))), search_pattern


def _get_modified_recids_invenio2(from_date):
    """Get record ids for Invenio 2."""
    from invenio.legacy.search_engine import search_pattern
    from invenio.modules.records.models import Record

    date = datetime.datetime.strptime(from_date, '%Y-%m-%d %H:%M:%S')
    return set((x[0] for x in Record.query.filter(
        Record.modification_date >= date).values(Record.id))), search_pattern


def get_modified_recids(from_date):
    """Get modified recids."""
    try:
        return _get_modified_recids_invenio12(from_date)
    except ImportError:
        return _get_modified_recids_invenio2(from_date)


def get_record_revisions(recid, from_date):
    """Get record revisions."""
    try:
        from invenio.dbquery import run_sql
    except ImportError:
        from invenio.legacy.dbquery import run_sql

    return run_sql(
        'SELECT job_date, marcxml '
        'FROM hstRECORD WHERE id_bibrec = %s AND job_date >= %s '
        'ORDER BY job_date ASC', (recid, from_date), run_on_slave=True)


def dump_record_json(marcxml):
    """Dump JSON of record."""
    try:
        from invenio.modules.records.api import Record
        d = Record.create(marcxml, 'marc')
        return d.dumps(clean=True)
    except ImportError:
        return None


def get(query, from_date, **kwargs):
    """Get recids matching query and with changes."""
    recids, search_pattern = get_modified_recids(from_date)
    recids = recids.union(get_modified_bibdoc_recids(from_date))

    if query:
        recids = recids.intersection(
            set(search_pattern(p=query.encode('utf-8'))))

    return len(recids), recids


def dump(recid, from_date, with_json=True, latest_only=False, **kwargs):
    """Dump MARCXML and JSON representation of a record.

    :param recid: Record identifier
    :param from_date: Dump only revisions from this date onwards.
    :param json_converter: Function to convert the record to JSON
    :returns: List of versions of the record.
    """
    date = datetime.datetime.strptime(from_date, '%Y-%m-%d %H:%M:%S')

    # Grab latest only
    if latest_only:
        revision_iter = [get_record_revisions(recid, from_date)[-1]]
    else:
        revision_iter = get_record_revisions(recid, from_date)

    # Dump revisions
    record_dump = dict(record=[], files=[], recid=recid)

    for revision_date, revision_marcxml in revision_iter:
        marcxml = zlib.decompress(revision_marcxml)
        record_dump['record'].append(dict(
            modification_datetime=datetime_toutc(revision_date).isoformat(),
            marcxml=marcxml,
            json=dump_record_json(marcxml) if with_json else None,
        ))

    record_dump['files'] = dump_bibdoc(recid, from_date)

    return record_dump



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Celery task for records migration."""

from __future__ import absolute_import, print_function

from celery import shared_task
from celery.utils.log import get_task_logger

from .errors import DepositMultipleRecids, DepositRecidDoesNotExist, \
    DepositSIPUserDoesNotExist

logger = get_task_logger(__name__)


@shared_task()
def load_deposit(data):
    """Load the raw JSON dump of the Deposition.

    Uses Record API in order to bypass all Deposit-specific initialization,
    which are to be done after the final stage of deposit migration.

    :param data: Dictionary containing deposition data.
    :type data: dict
    """
    deposit, dep_pid = create_record_and_pid(data)
    deposit = create_files_and_sip(deposit, dep_pid)


def create_record_and_pid(data):
    """Create the deposit record metadata and persistent identifier.

    :param data: Raw JSON dump of the deposit.
    :type data: dict
    :returns: A deposit object and its pid
    :rtype: (`invenio_records.api.Record`,
             `invenio_pidstore.models.PersistentIdentifier`)
    """
    from invenio_db import db
    from invenio_records.api import Record
    from invenio_pidstore.models import PersistentIdentifier, PIDStatus

    deposit = Record.create(data=data)
    depid = deposit['_p']['id']
    pid = PersistentIdentifier.create(
        pid_type='depid',
        pid_value=str(depid),
        object_type='rec',
        object_uuid=str(deposit.id),
        status=PIDStatus.REGISTERED
    )
    deposit.commit()
    db.session.commit()
    return deposit, pid


def create_files_and_sip(deposit, dep_pid):
    """Create deposit Bucket, Files and SIPs."""
    from invenio_pidstore.errors import PIDDoesNotExistError
    from invenio_pidstore.models import PersistentIdentifier
    from invenio_sipstore.errors import SIPUserDoesNotExist
    from invenio_sipstore.models import SIP, RecordSIP, SIPFile
    from invenio_files_rest.models import Bucket, FileInstance, ObjectVersion
    from invenio_records_files.models import RecordsBuckets
    from invenio_db import db
    buc = Bucket.create()
    recbuc = RecordsBuckets(record_id=deposit.id, bucket_id=buc.id)
    db.session.add(recbuc)
    deposit.setdefault('_deposit', dict())
    deposit.setdefault('_files', list())
    files = deposit.get('files', [])
    sips = deposit.get('sips', [])
    recid = None

    if sips:
        recids = [int(sip['metadata']['recid']) for sip in sips]
        if len(set(recids)) > 1:
            logger.error('Multiple recids ({recids}) found in deposit {depid}'
                         ' does not exists.'.format(recids=recids,
                                                    depid=dep_pid.pid_value))
            raise DepositMultipleRecids(dep_pid.pid_value, list(set(recids)))
        elif recids:  # If only one recid
            recid = recids[0]

    # Store the path -> FileInstance mappings for SIPFile creation later
    dep_file_instances = list()

    for file_ in files:
        fi = FileInstance.create()
        fi.set_uri(file_['path'], file_['size'], file_['checksum'])
        ov = ObjectVersion.create(buc, file_['name'], _file_id=fi.id)
        file_meta = dict(
            bucket=str(buc.id),
            key=file_['name'],
            checksum=file_['checksum'],
            size=file_['size'],
            version_id=str(ov.version_id),
        )
        deposit['_files'].append(file_meta)
        dep_file_instances.append((file_['path'], fi))

    for idx, sip in enumerate(sips):
        agent = None
        user_id = None
        if sip['agents']:
            agent = dict(
                ip_address=sip['agents'][0].get('ip_address', ""),
                email=sip['agents'][0].get('email_address', ""),
            )
            user_id = sip['agents'][0]['user_id']
        content = sip['package']
        sip_format = 'marcxml'
        try:
            sip = SIP.create(sip_format,
                             content,
                             user_id=user_id,
                             agent=agent)
        except SIPUserDoesNotExist:
            logger.exception('User ID {user_id} referred in deposit {depid} '
                             'does not exists.'.format(
                                 user_id=user_id, depid=dep_pid.pid_value))
            raise DepositSIPUserDoesNotExist(dep_pid.pid_value, user_id)

        # If recid was found, attach it to SIP
        # TODO: This is always uses the first recid, as we quit if multiple
        # recids are found in the sips information
        if recid:
            try:
                pid = PersistentIdentifier.get(pid_type='recid',
                                               pid_value=recid)
                record_sip = RecordSIP(sip_id=sip.id, pid_id=pid.id)
                db.session.add(record_sip)
            except PIDDoesNotExistError:
                logger.exception('Record {recid} referred in '
                                 'Deposit {depid} does not exists.'.format(
                                     recid=recid, depid=dep_pid.pid_value))
                raise DepositRecidDoesNotExist(dep_pid.pid_value, recid)
        if idx == 0:
            for fp, fi in dep_file_instances:
                sipf = SIPFile(sip_id=sip.id, filepath=fp, file_id=fi.id)
                db.session.add(sipf)
    deposit.commit()
    db.session.commit()
    return deposit



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio-Deposit migration."""

from __future__ import absolute_import, print_function

import datetime
import json
from itertools import islice

from .utils import datetime_toutc


def dt2utc_timestamp(dt):
    """Serialize the datetime to UTC timestamp."""
    return datetime_toutc(dt).isoformat()


def default_serializer(o):
    """Default serializer for json."""
    defs = (
        ((datetime.date, datetime.time),
         lambda x: x.isoformat(), ),
        ((datetime.datetime, ),
         lambda x: dt2utc_timestamp(x), ),
    )
    for types, fun in defs:
        if isinstance(o, types):
            return fun(o)


def get(query, from_date, limit=0, **kwargs):
    """Get deposits."""
    from invenio.modules.deposit.models import Deposition
    dep_generator = Deposition.get_depositions()
    total_depids = 1  # Count of depositions is hard to determine

    # If limit provided, serve only first n=limit items
    if limit > 0:
        dep_generator = islice(dep_generator, limit)
        total_depids = limit
    return total_depids, dep_generator


def dump(deposition, from_date, with_json=True, latest_only=False, **kwargs):
    """Dump the deposition object as dictionary."""
    # Serialize the __getstate__ and fall back to default serializer
    dep_json = json.dumps(deposition.__getstate__(),
                          default=default_serializer)
    dep_dict = json.loads(dep_json)
    dep_dict['_p'] = {}
    dep_dict['_p']['id'] = deposition.id
    dep_dict['_p']['created'] = dt2utc_timestamp(deposition.created)
    dep_dict['_p']['modified'] = dt2utc_timestamp(deposition.modified)
    dep_dict['_p']['user_id'] = deposition.user_id
    dep_dict['_p']['state'] = deposition.state
    dep_dict['_p']['has_sip'] = deposition.has_sip()
    dep_dict['_p']['submitted'] = deposition.submitted
    return dep_dict



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Community dump functions."""

from __future__ import absolute_import, print_function


def get(*args, **kwargs):
    """Get communities."""
    from invenio.modules.communities.models import FeaturedCommunity
    q = FeaturedCommunity.query
    return q.count(), q.all()


def dump(fc, from_date, with_json=True, latest_only=False, **kwargs):
    """Dump the community object as dictionary.

    :param fc: Community featuring to be dumped.
    :type fc: `invenio_communities.models.FeaturedCommunity [Invenio2.x]`
    :returns: Community serialized to dictionary.
    :rtype: dict
    """
    return dict(id=fc.id,
                id_community=fc.id_community,
                start_date=fc.start_date.isoformat())



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Bibdocfile metadata dumper."""

from __future__ import absolute_import, print_function

import datetime

import click

from .utils import datetime_toutc


def _get_recids_invenio12(from_date):
    """Get BibDocs for Invenio 1."""
    from invenio.dbquery import run_sql
    return (id[0] for id in run_sql(
        'select id_bibrec from '
        'bibrec_bibdoc as r join bibdoc as d on r.id_bibdoc=d.id '
        'where d.modification_date >=%s',
        (from_date, ), run_on_slave=True))


def _get_recids_invenio2(from_date):
    """Get BibDocs for Invenio 2."""
    from invenio.legacy.dbquery import run_sql
    return (id[0] for id in run_sql(
        'select id_bibrec from '
        'bibrec_bibdoc as r join bibdoc as d on r.id_bibdoc=d.id '
        'where d.modification_date >=%s',
        (from_date, ), run_on_slave=True))


def get_modified_bibdoc_recids(from_date):
    """Get bibdocs."""
    try:
        return set(_get_recids_invenio12(from_date))
    except ImportError:
        return set(_get_recids_invenio2(from_date))


def _import_bibdoc():
    """Import BibDocFile."""
    try:
        from invenio.bibdocfile import BibRecDocs, BibDoc
    except ImportError:
        from invenio.legacy.bibdocfile.api import BibRecDocs, BibDoc
    return BibRecDocs, BibDoc


def dump_bibdoc(recid, from_date, **kwargs):
    """Dump all BibDoc metadata.

    :param docid: BibDoc ID
    :param from_date: Dump only BibDoc revisions newer than this date.

    :returns: List of version of the BibDoc formatted as a dict
    """
    BibRecDocs, BibDoc = _import_bibdoc()

    bibdocfile_dump = []

    date = datetime.datetime.strptime(from_date, '%Y-%m-%d %H:%M:%S')
    for bibdoc in BibRecDocs(recid).list_bibdocs():
        for version in bibdoc.list_versions():
            bibdoc_version = bibdoc.list_version_files(version)
            for f in bibdoc_version:
                if f.is_icon() or f.md < date:
                    # Don't care about icons
                    # Don't care about files not modified since from_date
                    continue
                bibdocfile_dump.append(dict(
                    bibdocid=f.get_bibdocid(),
                    checksum=f.get_checksum(),
                    comment=f.get_comment(),
                    copyright=(
                        f.get_copyright() if hasattr(f, 'get_copyright')
                        else None),
                    creation_date=datetime_toutc(f.cd).isoformat(),
                    description=f.get_description(),
                    encoding=f.encoding,
                    etag=f.etag,
                    flags=f.flags,
                    format=f.get_format(),
                    full_name=f.get_full_name(),
                    full_path=f.get_full_path(),
                    hidden=f.hidden,
                    license=(
                        f.get_license()if hasattr(f, 'get_license') else None),
                    modification_date=datetime_toutc(f.md).isoformat(),
                    name=f.get_name(),
                    mime=f.mime,
                    path=f.get_path(),
                    recid=f.get_recid(),
                    recids_doctype=f.recids_doctypes,
                    size=f.get_size(),
                    status=f.get_status(),
                    subformat=f.get_subformat(),
                    superformat=f.get_superformat(),
                    type=f.get_type(),
                    url=f.get_url(),
                    version=f.get_version(),
                ))

    return bibdocfile_dump


def get_check():
    """Get bibdocs to check."""
    try:
        from invenio.dbquery import run_sql
    except ImportError:
        from invenio.legacy.dbquery import run_sql

    return (
        run_sql('select count(id) from bibdoc', run_on_slave=True)[0][0],
        [id[0] for id in run_sql('select id from bibdoc', run_on_slave=True)],
    )


def check(id_):
    """Check bibdocs."""
    BibRecDocs, BibDoc = _import_bibdoc()

    try:
        BibDoc(id_).list_all_files()
    except Exception:
        click.secho("BibDoc {0} failed check.".format(id_), fg='red')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

from invenio_pidstore.fetchers import FetchedPID

from invenio_migrator.records import RecordDump


def doi_fetcher(record_uuid, data):
    """Test fetcher."""
    if 'doi' in data:
        return FetchedPID(
            pid_type='doi', pid_value=data['doi'], provider=None
        )
    return None


def test_prepare_data_marcxml(records_json):
    """Test prepare data from marcxml."""
    d = RecordDump(records_json[0])
    d.prepare_revisions()
    assert len(d.revisions) == 1
    record = d.revisions[0][1]
    assert record['title_statement']['title']

    d = RecordDump(records_json[0], latest_only=True)
    d.prepare_revisions()
    assert len(d.revisions) == 1
    record = d.revisions[0][1]
    assert record['title_statement']['title']


def test_prepare_data_json(records_json):
    """Test prepare data from json."""
    d = RecordDump(records_json[0], source_type='json')
    d.prepare_revisions()
    assert len(d.revisions) == 1
    record = d.revisions[0][1]
    assert record['title']

    d = RecordDump(records_json[0], source_type='json', latest_only=True)
    d.prepare_revisions()
    assert len(d.revisions) == 1
    record = d.revisions[0][1]
    assert record['title']


def test_get_files(records_json):
    """Test get files."""
    d = RecordDump(records_json[0])
    d.prepare_files()
    assert len(d.files) == 1
    assert 'CERN_openlab_Parin_Porecha.pdf' in d.files


def test_record_property(app, db, records_json, record_db):
    """Test get files."""
    d = RecordDump(records_json[0])
    assert d.record


def test_record_property_no_record(app, db, records_json):
    """Test get files."""
    d = RecordDump(records_json[0])
    assert d.record is None


def test_pop_first_revision(records_json):
    """Test get files."""
    d = RecordDump(records_json[0])
    d.prepare_revisions()
    assert len(d.revisions) == 1
    assert d.pop_first_revision()
    assert len(d.revisions) == 0


def test_prepare_pids(app, db, records_json, record_db):
    """Test get files."""
    d = RecordDump(records_json[0], source_type='json')
    d.prepare_revisions()
    d.prepare_pids()
    assert len(d.pids) == 0

    d = RecordDump(
        records_json[0], source_type='json', pid_fetchers=[doi_fetcher])
    d.prepare_revisions()
    d.prepare_pids()
    assert len(d.pids) == 1


def test_missing_pids_pid_exist(app, db, records_json, record_pid):
    """Test get files."""
    d = RecordDump(
        records_json[0], source_type='json', pid_fetchers=[doi_fetcher])
    d.prepare_revisions()
    d.prepare_pids()
    assert len(d.pids) == 1
    assert len(d.missing_pids) == 0


def test_missing_pids(app, db, records_json, record_db):
    """Test get files."""
    d = RecordDump(
        records_json[0], source_type='json', pid_fetchers=[doi_fetcher])
    d.prepare_revisions()
    d.prepare_pids()
    assert len(d.pids) == 1
    assert len(d.missing_pids) == 1


def test_is_deleted(records_json):
    """Test get files."""
    d = RecordDump(records_json[0])
    d.prepare_revisions()
    assert not d.is_deleted()

    d = RecordDump(
        {'record': [{
            'json': {'collections': ['deleted']},
            'modification_datetime': '2014-10-10T10:10:10Z'
        }]},
        source_type='json'
    )
    d.prepare_revisions()
    assert d.is_deleted()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

r"""Invenio-Communities migration.

Invenio-Communities
-------------------

Dumping data from Legacy
~~~~~~~~~~~~~~~~~~~~~~~~

Dump the models ``User``, ``Community`` and ``FeaturedCommunity``:

.. code-block:: console

   $ inveniomigrator dump users
   $ inveniomigrator dump communities
   $ inveniomigrator dump featured

Prepare a tarbar of community logos:

.. code-block:: console

   $ cd /(invenio instance path)/static/media
   $ tar cvf communities.tar ./communities

This will result in the output files:

- ``users_dump_*.json``
- ``communities_dump_*.json``
- ``featured_dump_*.json``
- ``communities.tar``

Loading data
~~~~~~~~~~~~

Extract the tarball ``communities.tar`` to a ``communities/`` directory:

.. code-block:: console

   $ tar xvf ./communities.tar

Initialize the communities files bucket:

.. code-block:: console

   $ invenio communities init

Load the users, communities and community featurings. Last parameter of
``invenio dumps loadcommunities`` should point to the directory containing
the community logos.

.. code-block:: console

   $ for f in ./users_dump_*.json; do invenio dumps loadusers $f; done
   $ for f in ./communities_dump_*.json; do invenio dumps loadcommunities $f \
           ./communities/; done
   $ for f in ./featured_dump_*.json; do invenio dumps loadfeatured $f; done
"""

from __future__ import absolute_import, print_function

from .utils import dt2iso_or_empty


def get(*args, **kwargs):
    """Get communities."""
    from invenio.modules.communities.models import Community
    q = Community.query
    return q.count(), q.all()


def dump(c, from_date, with_json=True, latest_only=False, **kwargs):
    """Dump the community object as dictionary.

    :param c: Community to be dumped.
    :type c: `invenio.modules.communities.models.Community`
    :returns: Community serialized to dictionary.
    :rtype: dict
    """
    return dict(id=c.id,
                id_user=c.id_user,
                title=c.title,
                description=c.description,
                page=c.page,
                curation_policy=c.curation_policy,
                last_record_accepted=dt2iso_or_empty(c.last_record_accepted),
                logo_ext=c.logo_ext,
                logo_url=c.logo_url,
                ranking=c.ranking,
                fixed_points=c.fixed_points,
                created=c.created.isoformat(),
                last_modified=c.last_modified.isoformat(),
                id_collection_provisional=c.id_collection_provisional,
                id_oairepository=c.id_oairepository,
                id_collection=c.id_collection)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

import pytest
from invenio_pidstore.resolver import Resolver
from invenio_records.api import Record
from invenio_records.models import RecordMetadata
from invenio_records_files.api import Record as RecordFiles
from invenio_sipstore.models import SIP, RecordSIP, SIPFile

from invenio_migrator.tasks.deposit import load_deposit
from invenio_migrator.tasks.errors import DepositMultipleRecids, \
    DepositRecidDoesNotExist, DepositSIPUserDoesNotExist


def test_deposit_load(dummy_location, deposit_user, deposit_record_pid):
    """Test the deposit loading function."""
    dep1 = dict(sips=[dict(metadata=dict(recid='10'),
                           agents=[dict(user_id=1), ],
                           package='Foobar'), ],
                _p=dict(id='1'))
    dep2 = dict(sips=[dict(metadata=dict(recid='50'),
                           agents=[dict(user_id=1), ],
                           package='Foobar'), ],
                _p=dict(id='2'))
    dep3 = dict(sips=[dict(metadata=dict(recid='10'),
                           agents=[dict(user_id=5), ],
                           package='Foobar'), ],
                _p=dict(id='3'))
    dep4 = dict(sips=[dict(metadata=dict(recid='10'),
                           agents=[dict(user_id=5), ],
                           package='Foobar'),
                      dict(metadata=dict(recid='11'),
                           agents=[dict(user_id=5), ],
                           package='Foobar'), ],
                _p=dict(id='4'))
    load_deposit(dep1)
    pytest.raises(DepositRecidDoesNotExist, load_deposit, dep2)
    pytest.raises(DepositSIPUserDoesNotExist, load_deposit, dep3)
    pytest.raises(DepositMultipleRecids, load_deposit, dep4)


def test_deposit_load_task(dummy_location, deposit_dump, deposit_user,
                           deposit_record_pid):
    """Test the deposit loading task."""

    # Create a user and a record with PID corresponding with test deposit data
    assert RecordMetadata.query.count() == 1
    for dep in deposit_dump:
        load_deposit.delay(dep)
    assert RecordMetadata.query.count() == 2
    res = Resolver(pid_type='depid', object_type='rec',
                   getter=Record.get_record)
    dep_pid, dep_rec = res.resolve('1')
    assert '_deposit' in dep_rec
    assert '_files' in dep_rec
    sip = SIP.query.one()
    assert sip.user_id == deposit_user.id
    rsip = RecordSIP.query.one()
    assert rsip.pid_id == deposit_record_pid.id
    assert rsip.sip_id == sip.id

    # Test RecordsFiles API
    res = Resolver(pid_type='depid', object_type='rec',
                   getter=RecordFiles.get_record)
    dep_pid, dep_recbucket = res.resolve('1')
    files = list(dep_recbucket.files)
    assert files[0]['key'] == 'bazbar.pdf'
    assert files[0]['size'] == 12345
    assert files[0]['checksum'] == "00000000000000000000000000000000"
    assert files[0]['bucket']
    assert SIPFile.query.count() == 1



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Celery task for records migration."""

from __future__ import absolute_import, print_function

from celery import shared_task
from invenio_db import db

from ..proxies import current_migrator


@shared_task()
def import_record(data, source_type=None, latest_only=False):
    """Migrate a record from a migration dump.

    :param data: Dictionary for representing a single record and files.
    :param source_type: Determines if the MARCXML or the JSON dump is used.
        Default: ``marcxml``.
    :param latest_only: Determine is only the latest revision should be loaded.
    """
    source_type = source_type or 'marcxml'
    assert source_type in ['marcxml', 'json']

    recorddump = current_migrator.records_dump_cls(
        data,
        source_type=source_type,
        pid_fetchers=current_migrator.records_pid_fetchers,
    )
    try:
        record = current_migrator.records_dumploader_cls.create(recorddump)
        db.session.commit()
    except Exception:
        db.session.rollback()
        raise

    return str(record.id)



# -*- coding: utf-8 -*-

# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

""""Helper methods."""

from __future__ import absolute_import, print_function

from dateutil.parser import parse as iso2dt


def iso2dt_or_none(iso_dt):
    """Turn ISO-formatted date into datetime (None if 'iso_dt' is empty).

    :param iso_dt: ISO-formatted date
    :type iso_dt: str
    :returns: datetime object or None
    :rtype: datetime
    """
    return iso2dt(iso_dt) if iso_dt else None


def logo_ext_wash(logo_ext):
    """Handle the legacy logo extensions.

    Examples:
    None -> None
    'png' -> 'png'
    '0' -> None
    '.jpg' -> 'jpg'

    :param logo_ext: Community logo extension.
    :type logo_ext: string or None
    :returns: Washed logo extension.
    :rtype: string or None
    """
    if logo_ext is None:
        return None
    elif logo_ext == '0':
        return None
    elif logo_ext.startswith('.'):
        return logo_ext[1:]
    else:
        return logo_ext



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Errors for migration tasks."""

from __future__ import absolute_import, print_function


class DepositError(Exception):
    """Base class for Deposit migration errors."""

    def __init__(self, pid, *args, **kwargs):
        """Initialize exception.

        :param pid: PersistentIdentifier of the Deposit metadata.
        :type pid: `invenio_pidstore.models.PersistentIdentifier`
        """
        self.pid = pid
        super(DepositError, self).__init__(*args, **kwargs)

    def __str__(self):
        """String representation of the error."""
        return "PID: {pid}".format(pid=self.pid)


class DepositRecidDoesNotExist(DepositError):
    """Deposit record PID does not exist."""

    def __init__(self, pid, recid, *args, **kwargs):
        """Initialize exception."""
        self.recid = recid
        super(DepositRecidDoesNotExist, self).__init__(pid, *args, **kwargs)

    def __str__(self):
        """String representation of the error."""
        return super(DepositRecidDoesNotExist, self).__str__() + \
            ", recid: {recid}".format(recid=self.recid)


class DepositMultipleRecids(DepositError):
    """Deposit metadata is referencing multiple recids."""

    def __init__(self, pid, recids, *args, **kwargs):
        """Initialize exception."""
        self.recids = recids
        super(DepositMultipleRecids, self).__init__(pid, *args, **kwargs)

    def __str__(self):
        """String representation of the error."""
        return super(DepositRecidDoesNotExist, self).__str__() + \
            ", recids: {recids}".format(recids=self.recids)


class DepositSIPUserDoesNotExist(DepositError):
    """Deposit SIP user does not exist."""

    def __init__(self, pid, user_id, *args, **kwargs):
        """Initialize exception."""
        self.user_id = user_id
        super(DepositSIPUserDoesNotExist, self).__init__(pid, *args, **kwargs)

    def __str__(self):
        """String representation of the error."""
        return super(DepositRecidDoesNotExist, self).__str__() + \
            ", user_id: {user_id}".format(user_id=self.user_id)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Module tests."""

from __future__ import absolute_import, print_function

from flask import Flask
from flask_cli import FlaskCLI

from invenio_migrator.ext import InvenioMigrator


class Dump(object):
    """Test class."""


class DumpLoader(object):
    """Test class."""


def fetcher(arg1, arg2):
    """Test method."""


def post_task():
    """Test method."""


def test_version():
    """Test version import."""
    from invenio_migrator import __version__
    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask('testapp')
    FlaskCLI(app)
    ext = InvenioMigrator(app)
    assert 'invenio-migrator' in app.extensions

    app = Flask('testapp')
    FlaskCLI(app)
    ext = InvenioMigrator()
    assert 'invenio-migrator' not in app.extensions
    ext.init_app(app)
    assert 'invenio-migrator' in app.extensions


def test_imp():
    """Test extension initialization."""
    app = Flask('testapp')
    FlaskCLI(app)
    app.config.update(dict(
        MIGRATOR_RECORDS_DUMP_CLS='test_invenio_migrator:Dump',
        MIGRATOR_RECORDS_DUMPLOADER_CLS='test_invenio_migrator:DumpLoader',
        MIGRATOR_RECORDS_PID_FETCHERS=[
            'test_invenio_migrator:fetcher'
        ],
        MIGRATOR_RECORDS_POST_TASK='test_invenio_migrator:post_task'
    ))
    ext = InvenioMigrator(app)
    assert ext.records_dump_cls == Dump
    assert ext.records_dumploader_cls == DumpLoader
    assert ext.records_pid_fetchers == [fetcher]
    assert ext.records_post_task == post_task
